'use strict';

var _testUtils = require('@vue/test-utils');

var _vue = require('vue');

var _vue2 = _interopRequireDefault(_vue);

var _ = require('..');

var _2 = _interopRequireDefault(_);

var _utils = require('@/tests/utils');

var _focusTest = require('../../../tests/shared/focusTest');

var _focusTest2 = _interopRequireDefault(_focusTest);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var getMentions = _2['default'].getMentions;


function $$(className) {
  return document.body.querySelectorAll(className);
}

function triggerInput(wrapper) {
  var text = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  wrapper.find('textarea').element.value = text;
  wrapper.find('textarea').element.selectionStart = text.length;
  wrapper.find('textarea').trigger('keydown');
  wrapper.find('textarea').trigger('change');
  wrapper.find('textarea').trigger('keyup');
}

describe('Mentions', function () {
  beforeAll(function () {
    jest.useFakeTimers();
  });

  afterAll(function () {
    jest.useRealTimers();
  });

  it('getMentions', function () {
    var mentions = getMentions('@light #bamboo cat', { prefix: ['@', '#'] });
    expect(mentions).toEqual([{
      prefix: '@',
      value: 'light'
    }, {
      prefix: '#',
      value: 'bamboo'
    }]);
  });

  it('focus', function () {
    var onFocus = jest.fn();
    var onBlur = jest.fn();

    var wrapper = (0, _testUtils.mount)({
      render: function render() {
        var h = arguments[0];

        return h(_2['default'], {
          on: {
            'focus': onFocus,
            'blur': onBlur
          }
        });
      }
    });
    wrapper.find('textarea').trigger('focus');
    expect(wrapper.find('.ant-mentions').classes('ant-mentions-focused')).toBeTruthy();
    expect(onFocus).toHaveBeenCalled();

    wrapper.find('textarea').trigger('blur');
    jest.runAllTimers();
    expect(wrapper.classes()).not.toContain('ant-mentions-focused');
    expect(onBlur).toHaveBeenCalled();
  });

  it('loading', function (done) {
    var wrapper = (0, _testUtils.mount)({
      render: function render() {
        var h = arguments[0];

        return h(_2['default'], {
          attrs: { loading: true }
        });
      }
    }, { sync: false });
    triggerInput(wrapper, '@');
    _vue2['default'].nextTick(function () {
      (0, _testUtils.mount)({
        render: function render() {
          return wrapper.find({ name: 'Trigger' }).vm.getComponent();
        }
      }, { sync: false });
      _vue2['default'].nextTick(function () {
        expect($$('.ant-mentions-dropdown-menu-item').length).toBeTruthy();
        expect($$('.ant-spin')).toBeTruthy();
        done();
      });
    });
  });

  (0, _focusTest2['default'])(_2['default']);
});